@@ -1,19 +1,29 @@
 """YoLink mqtt client."""
+
 import asyncio
-import asyncio_mqtt as aiomqtt
 import logging
 from typing import Any
-from pydantic import ValidationError
+import aiomqtt
+
+try:
+    from pydantic.v1 import ValidationError
+except ImportError:
+    from pydantic import ValidationError
+
 from .auth_mgr import YoLinkAuthMgr
 from .const import (
-    YOLINK_API_MQTT_BROKER,
-    YOLINK_API_MQTT_BROKER_PORT,
     ATTR_DEVICE_SMART_REMOTER,
+    ATTR_DEVICE_WATER_DEPTH_SENSOR,
+    ATTR_DEVICE_WATER_METER_CONTROLLER,
 )
 from .device import YoLinkDevice
 from .message_listener import MessageListener
 from .model import BRDP
-from .message_resolver import smart_remoter_message_resolver
+from .message_resolver import (
+    smart_remoter_message_resolve,
+    water_depth_sensor_message_resolve,
+    water_meter_sensor_message_resolver,
+)
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -22,11 +32,19 @@ class YoLinkMqttClient:
     """YoLink mqtt client."""
 
     def __init__(
-        self, auth_manager: YoLinkAuthMgr, home_devices: dict[str, YoLinkDevice]
+        self,
+        auth_manager: YoLinkAuthMgr,
+        endpoint: str,
+        broker_host: str,
+        broker_port: int,
+        home_devices: dict[str, YoLinkDevice],
     ) -> None:
+        self._auth_mgr = auth_manager
+        self._endpoint = endpoint
+        self._broker_host = broker_host
+        self._broker_port = broker_port
         self._home_topic = None
         self._message_listener = None
-        self._auth_mgr = auth_manager
         self._home_devices = home_devices
         self._running = False
         self._listener_task = None
@@ -45,25 +63,38 @@ class YoLinkMqttClient:
         while self._running:
             try:
                 async with aiomqtt.Client(
-                    hostname=YOLINK_API_MQTT_BROKER,
-                    port=YOLINK_API_MQTT_BROKER_PORT,
+                    hostname=self._broker_host,
+                    port=self._broker_port,
                     username=self._auth_mgr.access_token(),
                     password="",
                     keepalive=50,
                 ) as client:
-                    async with client.messages() as messages:
-                        await client.subscribe(self._home_topic)
-                        async for message in messages:
-                            self._process_message(message)
+                    _LOGGER.info(
+                        "[%s] connecting to yolink mqtt broker.", self._endpoint
+                    )
+                    await client.subscribe(self._home_topic)
+                    _LOGGER.info("[%s] yolink mqtt client connected.", self._endpoint)
+                    async for message in client.messages:
+                        self._process_message(message)
             except aiomqtt.MqttError as mqtt_err:
-                _LOGGER.error("yolink mqtt client disconnected!")
+                _LOGGER.error(
+                    "[%s] yolink mqtt client disconnected!",
+                    self._endpoint,
+                    exc_info=True,
+                )
                 await asyncio.sleep(reconnect_interval)
                 if isinstance(mqtt_err, aiomqtt.MqttCodeError):
                     if mqtt_err.rc in [4, 5]:
-                        _LOGGER.error("token expired or invalid, acquire new one.")
+                        _LOGGER.error(
+                            "[%s] token expired or invalid, acquire new one.",
+                            self._endpoint,
+                        )
                         await self._auth_mgr.check_and_refresh_token()
+            except Exception:
+                _LOGGER.error("[%s] unexcept exception:", self._endpoint, exc_info=True)
 
     async def disconnect(self) -> None:
+        """UnRegister listener"""
         if self._listener_task is None:
             return
         self._listener_task.cancel()
@@ -87,7 +118,13 @@ class YoLinkMqttClient:
                     return
                 msg_event = msg_data.event.split(".")
                 msg_type = msg_event[len(msg_event) - 1]
-                if msg_type not in ["Report", "Alert", "StatusChange", "getState"]:
+                if msg_type not in [
+                    "Report",
+                    "Alert",
+                    "StatusChange",
+                    "getState",
+                    "setState",
+                ]:
                     return
                 device = self._home_devices.get(device_id)
                 if device is None:
@@ -108,5 +145,9 @@ class YoLinkMqttClient:
     def __resolve_message(self, device: YoLinkDevice, msg_data: dict[str, Any]) -> None:
         """Resolve device message."""
         if device.device_type == ATTR_DEVICE_SMART_REMOTER:
-            msg_data = smart_remoter_message_resolver(msg_data)
+            msg_data = smart_remoter_message_resolve(msg_data)
+        if device.device_type == ATTR_DEVICE_WATER_DEPTH_SENSOR:
+            msg_data = water_depth_sensor_message_resolve(msg_data, device.device_attrs)
+        if device.device_type == ATTR_DEVICE_WATER_METER_CONTROLLER:
+            msg_data = water_meter_sensor_message_resolver(msg_data)
         self._message_listener.on_message(device, msg_data)
